function [h, Di, beta, c, Q,NorQ] = OOMP_FFT(f, D, tol, No, ind,eex,option)
%OOMP Optimized Orthogonal Matching Pursuit 
%for trigonometric %dictionaries
%
% It creates an atomic decomposition of a signal with trigonometric dictioanries,
% using OOMP criterion. You can choose a  tolerance, the number of atoms to take in 
% or an initial subspace to influence the OOMP algorithm.  
% 
%  
% Usage:    [h, Di, beta, c ] = OMP(f, D, tol, No, ind,option);
%           [h, Di ] = OMP( f, D, tol );
%
% Inputs:
%   f       analyzing signal
%   D       dictionary of normalized atoms
%   tol     desired distance between f and its approximation. The routine will stop if
%           norm(f'-Dsub*(f*beta)')*sqrt(delta)<tol where delta=1, but can be changed to 
%           1/L, L is number of points  in a sample.
%   No      (optional) maximal number of atoms to choose, if the number of chosen atoms
%           equals to No, routine will stop (default No=size(D,2) 
%   ind     (optional) indices determining the initial subspace. 
%   option  1 for the dictionary
%           option=4 for Sin-Cos, option=2 for Cos option=3 for Sin dictionaries
%
% Outputs:
%   h       the approximated signal
%   Di      indices of 'k' selcted  atoms.
%   beta    biorthogonal atoms to the selected ones. 
%   c       coefficients of the atomic decomposition
%   Q       orthogonalized selected atoms. 
%  
% This implementation only works for trigonometric dictionaries, to reproduce results in the 
% paper below. It should be call with the script Example_I
% OMP for general dictionaries is available on http://www.nonlinear-approx.info/
%
% Reference:
%
% L. Rebollo-Neira, "Optimized Orthogonal Matching Pursuit Approach",
%   
name='OOMP_FFT';  %name of routine
% get inputs and setup parameters
[L,N]=size(D);
zmax=1;%number of reorthogonalizations
beta=[];
Re=f;
Range=1:N; %  
Di=[];     %  
%delta=1/L;  
delta=1;   
%if nargin<6 option=4;end
%if nargin<5 ind=[];end
%if nargin<4 No=N;end

NorQ=zeros(N,1);
if option==4 NorQ=zeros(N/2,2);end

numind=length(ind);

%atoms having smaller norm than tol1 are supposed be zero ones
tol1=1e-7; %1e-5
%threshold for coefficients
tol2=1e-10;   %0.0001  %1e-5

%tic;
%fprintf('\n%s is running for tol=%g, tol1=%g and tol2=%g.\n',name,tol,tol1,tol2);
%fprintf('tol  -> required precision, distance ||f-f_approx||\n')
%fprintf('tol1 -> atoms having smaller norm are supposed to be zero ones.\n');
%fprintf('tol2 -> algorithm stops if max(|<f,q>|/||q||)<= tol2.\n');
%===============================
% Main algorithm: at kth iteration
%===============================
H=min(No,N); %maximal number of function in sub-dictionary
for k=1:H    
  %finding of maximal coefficient
  c=zeros(1,N);cc=zeros(1,N);
  if k<=numind 
    [test,q]=ismember(ind(k),Range);
    if test~=1 error('Demanded index %d is out of dictionary',ind(k));end
  else 
%  for p=k:N, %it can be done without for
%     cc(p)=abs(Re*D(:,p));
%  end
%    cc2=abs(Re*D);
%    [max_c,q]=max(cc2);
    [max_c,q,scra]=fast_chose_trig_atom_oomp(Re,N,NorQ,eex,option);
    %stopping criterion (coefficient)
    if max_c<tol2 
      k=k-1;
      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
      break;
    end
  end  
% if q~=k
% D(:,[k q])=D(:,[q k]); % swap k-th and q-th columns
% Di([k q])=Di([q k]);
  Di=[Di q];
% end
  if k>1
%Q(:,k) is the orthogonalization of D(:,k)  w.r.t Q(:,1),..., Q(:,k-1) 
    for p=1:k-1
     Q(:,k)=D(:,q)-(Q(:,p)'*D(:,q))*Q(:,p); %orthogonalization
    end 
   %re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1) 
   for zi=1:zmax
    for p=1:k-1
     Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p); %re-orthogonalization
    end	   
   end
  end
  if k==1 Q(:,k)=D(:,q); end
  nork=norm(Q(:,k)); 
  Q(:,k)=Q(:,k)/nork; %normalization
  % compute biorthogonal vectors beta from 1 to k-1
  if k>1
    beta=beta-Q(:,k)*(D(:,q)'*beta)/nork; 
  end	
  beta(:,k)=Q(:,k)/nork; % kth biorthogonal function
  %
%%%%computes de residum%%%%%   
   Re=Re-f*Q(:,k)*Q(:,k)';
   NorQ=Auxi(NorQ,Q(:,k)',N,eex,option); 
   [gt1]=find(NorQ(:,1)==1);
   NorQ(gt1,1)=0; %sloppy to delate the already chosen atoms
  if option==4
    [gt2]=find(NorQ(:,2)==1);
     NorQ(gt2,2)=0; %sloppy to delate the already chosen atoms
  end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%
  %stopping criterion (distance)
  if (tol~= 0) & (norm(Re)*sqrt(delta) < tol) break;end;
end
c=f*beta; %coefficients
h=(f-Re);%approximation

%fprintf('From %d atoms in this dictionary %s has chosen %d atoms.\n',N,name,k);
%fprintf('\n%s lasted %g seconds\n',name,toc); 
%fprintf('\nNorm ||f-f_approx|| is %g. \n',normf);
% error testing orthogonality
%ErrorTest(Q(:,1:k));
% biorthogonality
%ErrorTest(D(:,Di),beta);

%Copyright (C) 2014 Laura REBOLLO-NEIRA
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
